<?php
/**
 * @file
 * Contains the ExpandCollapseFormatter class.
 */

namespace Drupal\expand_collapse_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * The expand collapse formatter.
 *
 * @FieldFormatter(
 *   id = "expand_collapse_formatter",
 *   module = "expand_collapse_formatter",
 *   label = @Translation("Expand collapse formatter"),
 *   field_types = {
 *     "string_long",
 *     "text_long"
 *   }
 * )
 */
class ExpandCollapseFormatter extends FormatterBase {
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'trim_length' => 300,
      'default_state' => 'collapsed',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = [];

    $element['trim_length'] = [
      '#title' => t('Trim length'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('trim_length'),
      '#min' => 1,
      '#required' => TRUE,
    ];

    $element['default_state'] = [
      '#title' => t('Default state'),
      '#type' => 'select',
      '#options' => [
        'collapsed' => t('Collapsed'),
        'expanded' => t('Expanded'),
      ],
      '#default_value' => $this->getSetting('default_state'),
      '#required' => TRUE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();
    $summary[] = t('Trim length: @trim_length', ['@trim_length' => $this->getSetting('trim_length')]);
    $summary[] = t('Default state: @default_state', ['@default_state' => $this->getSetting('default_state')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      // Render each element as markup.
      $element[$delta] = [
        '#theme' => 'expand_collapse_formatter',
        '#value' => $item->value,
        '#trim_length' => $this->getSetting('trim_length'),
        '#default_state' => $this->getSetting('default_state'),
        '#attached' => [
          'library' => [
            'expand_collapse_formatter/expand_collapse_formatter',
          ],
        ],
      ];
    }

    return $element;
  }

}
